

--[[
	EXAMPLE File for custom Agent Classes.
	Copy& Paste this Code into your agent class and modify it to your needs.
	
	
	>>>>BASIC INSTRUCTIONS<<<<
	
	AGENT / FILE NAME:
	Every agent has 2x names: The display name ('AGENT EXAMPLE') and the intern name ('agentexample')
	In your code you always use the intern file name
	The intern name has to match the file name.
	So if you change the internal name (in the first line: agents[<...>]), you have to change the filename too!
	It is good practice to make this name lower case and without spaces, to support all kinds of OS IO operations
	Note that this name also get concated with other file extensions in gfx/ or sfx/ folder for UTIL ICONS

	PARAMETERS:
	Most functions have data& pid as parameter. The pid is the player id and the data is pid specific!
	Every pid has his own data,
	The following is always true:
	- agent.current[pid] == 'agentexample', so you dont need to check if the player is currently the relevant agent
	
	UTIL / ABILITIES:
	Every util needs a few fields like the name, mana costs, etc.
	The functions (onTrigger) only get called when the user has sufficient mana, uses Left, no cooldown, etc.
	
	isTriggerable(data,pid) is optional. If you implement it:
	- it has to return true or false
	- will be called when the user triggered the ability. 
	- if it returns true, onTrigger will ALWAYS triggered right after it. (see agent.lua > agent.triggerUtil)
	Use it to check additional conditions (besides mana, cooldown, uses Left, etc.) when the user presses it.
	Example can be found in Veyne.lua, where the user CANT use his Tumble ability when the mouse cursor is to far away from the player

	Select one specific kind for your ability (kinds defined in conf.lua > conf.utilKind).
	There is DURATION, CLICK and TOGGLE.
	- Use DURATION, specify a duration and a onUntrigger(data, pid) function that will be called automaticly after the duration ended
	- Use CLICK, to just have a one-time event with onTrigger() and nothing more
	- Use TOGGLE, specify a onUntrigger(data, pid) function. Either onTrigger or onUntrigger get called depending on the current toggle state

	The third ability (ULTIMATE) has an additional ULT COST field, similiar to the original Valorant
	
	NOTES:
	- Every table field here needs to be set in a meaningful way and not nil! (Exception: isTriggerable(), onSpawn() can be nil)
	- Every table field you find here will get called or used automaticly (according to their description)
	- Example: Do NOT check if the player has enough mana for the current ability in onTrigger() 
	- You can add custom fields ofcourse, but those will not be called automaticly, you have to hook/call them manually
	- Maybe you want to check exisiting agent classes (in agents/*.lua) to get a better picture after reading this
	- This specific agent example essentially does nothing, but uses all varietys in ability design
	
	ADDITIONAL FILES:
	Go to gfx/valo/... and sfx/valo/... and create your own icons for your agent.
	Those HAVE to exist and must not be included in the transfer field.
	
	***Note that the agentname in the filenames have to match the intern name described above.***
	Create the servertransfer.lst automaticly with the debug menu to cover your new agent files
]]

agents["agentexample"] =
{
	name = "AGENT EXAMPLE", -- Ingame Display name
	health = 100,
	speed = -1,				-- 0 is default, a Valorant gimmik that the players walk just a bit slower than in CS
	desc = "Do not use me :C", 	-- Short Agent Description, no commas allowed in here duo to cs2d menus.
	
	
	
	
	--[[
		Put all your additional gfx/sfx in here, so the debug transferlist generator knows to include them!
		The key can be anything you like (glowPath), the value needs to be the full cs2droot path
	]]--
	transfers = {
		["glowPath"] = "gfx/sprites/flare2.bmp",
	},

	--[[ 
		Happens exactly once on startup. Do your hooks here!
		Use the custom hookers for convenience, like in "gnuk.lua"
		So you dont have to type "if (agent.current[pid] == 'agentexample')" everytime
	]]
	onInit = function(self)
		-- the 'self' param is just a shortcut to agents['agentexample'] for convenience
		-- so instead of using agents['agentexample'].myFunction(), you can use self.myFunction()
	end,
	
	
	--[[
		Optional onSpawn command that gets executed when the round begins (on END BUY TIME to be specific!)
		Use it similiar to the spawn hook, return weapons you like to equip (Example: gnuk.lua, agent spawns with claw to use special abilities)
	]]
	onSpawn = function(data, pid)
		--[[
			This gets called midround, but please dont use the equip command.
			Maybe in the future there will be agents that can respawn other agents (or similiar).
			To support that, we might want to use this function as a spawn hook in the future
		]]
		return 45; -- Equip a Laser, because I feel like DC today (IMBA& OP)
	end,
	
	
	
	--[[
		Called on endround and when the user selects this agent.
		Setup/Clear your data here. You can check for fields from the last round.
		Might be useful in scenarios like:
		The user used a DURATION ability and the round ended in between, you can clear things up here.
		
	]]--
	onSelect = function(data, pid)
		-- Clear stuff up from last round
		if (data.hasUsedAbility1) then
			msg2(pid,"You are still using the wrong agent class <.<")
			msg2(pid,"But hey, you used F2 Ability last round, I am very proud :3")
		end
	
		-- Brutally reset all data for this specific pid and murder remaining info from last round
		data = 1
		data = {}
	end,
	
	
	--[[
		All agent abilities.
		The keys have to be 1, 2 and 3 only, do not change these!
	]]
	utils = {
		-- First Ability (F2)
		[1] = {
			name = "Kalingo",			-- Name, duh
			desc = "BatchYaBingo",		-- Description
			manaCost = 69,				-- Mana Costs of this spell
			maxUse = 13,				-- How many times can this spell be casted per round?
			cooldown = 1500,			-- How much time has to pass for this spell to be castable again (in ms)?
			kind = "CLICK",				-- The kind, please read BASIC INSTRUCTIONS in the very top
			
			-- Long Agent Description
			explanation = {
				" [F2] Explanation'n'Stuff: This is a test file, not ment to be used",
				"Please dont select this agent class!",
				"Enjoy the other fully working agents :)",
				"Thiz is Script0r clazz, OK?!",
				"Pray0rz for Dark Cornors",
			},
			
			
			-- Optional field: see BASIC INSTRUCTIONS
			isTriggerable = function(data, pid)
				-- Can only be used when >90 hp
				local health = player(pid,"health")
				if (health > 90) then
					data.hp = health
					return true;
				end
				return false
			end,
			
			-- onTrigger: Always mandatory field, no matter what kind
			onTrigger = function(data, pid)
				-- Duo to isTriggerable: This will ONLY ever be called, when the player has >90 hp
				-- So "data.hp" is never nil here
				msg2(pid,data.hp.." HEHE")
				data.hasUsedAbility1 = true
				msg2(pid,"You used F2 Ability on TEST Class. OMG I SAID DONT USE IT")
			end,
			
		},
		-- Second Ability (F3), similiar to First Ability
		[2] = {
			name = "Oddworld",				
			desc = "Fiction or Reality?",	
			manaCost = 0,					-- NO mana cost
			maxUse = 9999,					-- unlimited uses
			cooldown = 2000,				-- Prevent spamming
			kind = "TOGGLE",				
			
			
			explanation = {
				" [F3] Explanation'n'Stuff: Stuff nobody reads",
				" And then everybody says 'this agent is dogpoo' in disgusting voice",
				" which leads to smart people trolling",
			},
			
			-- No isTriggerable, because we dont want to check anything additionally
			
			onTrigger = function(data, pid)
				data.ability2toggle = true
				msg2(pid,"ON")
			end,
			
			-- Since its a toggle kind, you need an onUntrigger
			onUntrigger = function(data, pid)
				data.ability2toggle = false
				msg2(pid,"OFF")
			end,
			
		},
		-- Third Ability (F4), associated as ULTIMATE, ULT.
		[3] = {
			
			name = "Script Kiddie Questions",	-- name, duh
			desc = "Are Dumb AF",				-- description
			manaCost = 100,						-- mana costs,
			maxUse = 1,							-- max usages per round?
			ultCost = 5,						-- ult costs!!! special for ult :)
			cooldown = 2000,					-- needs to be higher than duration
			duration = 1887,					-- when untrigger?
			kind = "DURATION",
			
			explanation = {
				"[F4] Super-ULT: Why bother",
				"Study your Agent - wise quote from xXx69OneShot187xXx",
				"You can stop laughin now :)",
			},
			
			-- optional field
			isTriggerable = function(data, pid)
				if (data.hasUsedAbility1) then
					return true;
				end
				return false
			end,
			
			onTrigger = function(data, pid)
				msg2(pid,"You ULTED! R U CRAZY?! SWAP AGENT MAN.")
				data.x = player(pid,"x")
				data.y = player(pid,"y")
				
				-- Draw some image
				local img = image(agents["agentexample"].transfers.glowPath,1,0,100+pid)
				imagecolor(img,255,255,255)
				imageblend(img,1)
				imagealpha(img,0.5)
				data.myImg = img;
				
				-- Example usage of the red action bar that some agents use
				agent.action.start(pid, agents["agentexample"].utils[3].duration)
			end,
			
			-- Since its an DURATION kind, you need an onUntrigger
			onUntrigger = function(data, pid)
				if (data.x == nil) then 
					-- fail-safe if the user ulted just before the round ended
					-- we dont want to accidently teleport the player in the start of new round
					-- note: on endround the data.x gets nil in onSelection()
					return 
				end 
				parse("setpos "..pid.." "..data.x.." "..data.y)
				msg2(pid,"Reseted your position on ULT End :) PS: Work4Answers my man...")
				
				-- Undraw image
				freeimage(data.myImg)
				data.myImg = nil
			end,
			
		},
	},
	
	
	

}




