--[[
	VALO 2D
	CC - Crowd Control
	Basic game mechanics
		- introduction to Crowd Control effects
		- use cc.trigger() to apply CC effects to someone
		note: applying the same CC effect to the same played will just extend the CC duration
		so if you cc.trigger(1, "NoMOV", 3000) and after 2 seconds again, the player will only be freezed for 5 seconds
	
]]--


if (not cc) then cc = {} end
--[[
	cc.active = {
		["Kind1"] = userdata1,
		["Kind2"] = userdata2, 
		...
		["Kindn"] = userdatan,
	}
	
	userdata depends on params of cc.trigger(); 
	if "boring-mode": userdata = true/nil, depending if the mentioned CC is active or not
	if "advanced-mode" (like knockback): userdata = cc.kinds["Kindn"].getData())
	
	your "mode" depends on the specific CC kind (Knockback is advanced, NoATK is boring)
		
	
	:*: userdatan will always be!
		- not nil when the CC is triggered (<=> cc.counter[pid] > 0)	( {true} or getData())
		- nil when the CC is untriggered (<=> cc.counter[pid] = 0) 		( nil )
]]
cc.active = {}
cc.counter = {}
-- local player info list

function cc.init()
	-- normal hooks! because these manage arrays and stuff
	hooker.normal:Set({
		{cc.onDie,"die"},
		{cc.onLeave,"leave"},
		{cc.onUpdate,"update"},
		{cc.onEndbuy,"startround"},
		{cc.onJoin,"join"},
	})
	
	-- hook timers automaticly
	
	for kind, ccs in pairs(cc.kinds) do
		for _,timeStr in pairs(conf.hookTimes) do
			
			if (cc.kinds[kind][timeStr]) then
				if (debug) then
					--^-print("********Hooking Time in "..kind.." / "..timeStr..", f("..tostring(cc.kinds[kind][timeStr]).." with typ("..tostring(hooker.cc) )
					hooker.time:Hook(cc.kinds[kind][timeStr], timeStr, hooker.cc, {name = kind})
					
				end
			end
		end
		
		
		if (cc.kinds[kind].onInit) then
			cc.kinds[kind].onInit(cc.kinds[kind])
		end
	end
end

--[[
	Hooks
]]

function cc.onEndbuy()
	for _,pid in pairs(player(0,"tableliving")) do
		cc.reset(pid)
	end
end

function cc.onJoin(pid)
	cc.reset(pid)
end

function cc.onDie(pid)
	cc.reset(pid)
end

function cc.onLeave(pid)
	cc.active[pid] = {}
	cc.counter[pid] = {}
end
function cc.reset(pid)
	--print("[CC] Resetting System for "..pid)
	if (not cc.counter[pid]) then
		cc.counter[pid] = {}
		cc.active[pid] = {}
	end
	for kind,_ in pairs(cc.kinds) do
		cc.counter[pid][kind] = 0
		cc.active[pid][kind] = nil
	end
end

--[[
	Call examples:
		cc.trigger(1, "NoMOV", 3000)
		cc.trigger(1, "NoATK", 2500)
		cc.trigger(1, "IGNITE", 2200)
		
		local knockbackData = {
			duration = 5000,
			stepAmount = 30,
			x = player(1,"x"),
			y = player(1,"y"),
			dx = 12,
			dy = 12,
			onWall = print
		}
		cc.trigger(1,"KNOCKBACK", knockbackData)
		
		
	ccdata:
		Usually just the duration,
		KNOCKBACK requires table:
		ccdata = {
			duration = <time in MS>
			stepAmount = <smoothness>
			x= <start pixel x>
			y= <start pixel y>
			dx=<direction pixel x>
			dy=<direction pixel y>
			onWall = <function(id) on wall hit>
		}
			
]]
function cc.trigger(pid, kind, ccdata)
	-- Decide user input mode
	local data
	
	if (type(ccdata) == "number") then -- If only duration was given
		 --> untrigger itself per duration and handle data by kind itself
		if (not cc.kinds[kind].getData) then
			data = true
		else
			data = cc.kinds[kind].getData(pid, cc.active[pid][kind])
		end
		timerEx(ccdata,"cc.untrigger",1,pid,kind)
	else
		-- If user gave more complex data (more than duration,e.g.Knockback) (-> force untrigger old)
		cc.untrigger_force(pid,kind)
		data = ccdata
	end
	
	-- Override data with user-decided data
	cc.active[pid][kind] = data
	
	-- Remember that I triggered it one more time
	cc.counter[pid][kind] = cc.counter[pid][kind] + 1
	
	-- Trigger CC
	if (cc.kinds[kind].onTrigger) then
		cc.kinds[kind].onTrigger(pid, data)
	end
	
	--if -(debug) then
		--^--print("CC Triggered: pid="..pid..", kind="..kind..", ccdata(duration)="..tostring(ccdata)..", kdata="..tostring(data))
	--end
	
end

function cc.onUpdate()
	for _, id in ipairs(player(0,"tableliving")) do
		for kind,currentCC in pairs(cc.kinds) do
			if (cc.kinds[kind].onUpdate) then
				if (not cc.active[id]) then return end
				if (cc.active[id][kind]) then
					cc.kinds[kind].onUpdate(id)
				end
			end
		end
		
	end
	
end

function cc.isAffected(pid, kind)
	return (cc.counter[pid][kind] > 0);
end

function cc.untrigger(pid,kind)
	--if (debug) then
	--	print("CC Untriggered: pid="..pid..", kind="..kind)
	--end
	
	cc.counter[pid][kind] = cc.counter[pid][kind] - 1
	
	if (cc.counter[pid][kind] <= 0) then
		cc.untrigger_force(pid, kind)
	end
end


--[[
	Will cleanse the player from mentioned CC, forcing onUntrigger() and counter=0
]]
function cc.untrigger_force(pid,kind)
	cc.counter[pid][kind] = 0
		--print("CC Untriggered: Last of his kind")
		if (cc.kinds[kind].onUntrigger) then
			cc.active[pid][kind] = cc.kinds[kind].onUntrigger(pid, cc.active[pid][kind])
			return
		end
		cc.active[pid][kind] = nil
	
end

--[[
	Example for player 13 is currently under NoATK and BURNING and generic syntax

	cc.active = {
		[13] = {
			[1] = true,
			[5] = true,
			... (each CC kind)
			...
		},
		[pid] = {
			[cc.kinds.id] = true/false
		}
		... (each pid currently affected)
		...
	}
]]




cc.init()